/* No representations are made concerning either the merchantability of
   this software or the suitability of this software for any particular
   purpose. It is provided "as is" without express or implied warranty
   of any kind.

   License to copy and use this software is granted provided that these
   notices are retained in any copies of any part of this documentation
   and/or software.
 */

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <windowsx.h>
#include "config.h"

int main(int argc, char* argv[])
{
	char buffer[80], *configFilename;
	FILE *infile;

	if (argc != 2) {
		// Had more than one non-option parameter
	    puts ("Usage: microlazeconfig configfile");
		return (0);
	}

	configFilename = argv[1];

	MicrolazeConfig config;

	// Try to open
	infile = fopen (configFilename, "r");
	if (infile == NULL)
		// Assume couldn't open because doesn't exist
		printf ("Creating new config file: %s\n", configFilename);
	else {
		// Only used infile to test for existence
		fclose (infile);

		printf ("Reading previous values from config file: %s\n", configFilename);
		if (config.loadFromFile (configFilename) == FALSE)
			// already printed error
			return (0);
	}
	
	puts ("\nEnter new values.  For [default] just hit enter.\n");

	printf ("Enter wavelength (IR, Green or UV) [%s]: ", config.wavelengthString());
	gets (buffer);
	if (*buffer)
		config.setWavelength (buffer);

	printf ("Enter energyrange (low or high) [%s]: ", config.energyRangeString());
	gets (buffer);
	if (*buffer)
		config.setEnergyRange (buffer);

	printf ("Enter energylevel [%d]: ", config.energyLevel());
	gets (buffer);
	if (*buffer)
		config.setEnergyLevel (buffer);

	printf ("Enter pulserate [%d]: ", config.pulseRate());
	gets (buffer);
	if (*buffer)
		config.setPulseRate (buffer);

	printf ("Enter markersizeX [%d]: ", config.markerSizeX());
	gets (buffer);
	if (*buffer)
		config.setMarkerSizeX (buffer);

	printf ("Enter markersizeY [%d]: ", config.markerSizeY());
	gets (buffer);
	if (*buffer)
		config.setMarkerSizeY (buffer);

	printf ("Enter scoreStepSize (mm) [%.3lf]: ", config.scoreStepSize());
	gets (buffer);
	if (*buffer)
		config.setScoreStepSize (buffer);

	printf ("Enter scoreHoldTime (sec) [%.3lf]: ", config.scoreHoldTime());
	gets (buffer);
	if (*buffer)
		config.setScoreHoldTime (buffer);

	printf ("Enter cutStepSize (mm) [%.3lf]: ", config.cutStepSize());
	gets (buffer);
	if (*buffer)
		config.setCutStepSize (buffer);

	printf ("Enter cutHoldTime (sec) [%.3lf]: ", config.cutHoldTime());
	gets (buffer);
	if (*buffer)
		config.setCutHoldTime (buffer);

	printf ("Enter cutFeedSpeed (1 to 100%) [%d]: ", config.cutFeedSpeed());
	gets (buffer);
	if (*buffer)
		config.setCutFeedSpeed (buffer);

	printf ("Enter distanceToRefocus (mm) (zero means don't refocus) [%.3lf]: ", config.distanceToRefocus());
	gets (buffer);
	if (*buffer)
		config.setDistanceToRefocus (buffer);

	printf ("Enter focusOffsetDistance (mm) (to move away from cut to refocus) [%.3lf]: ", config.focusOffcenterDistance());
	gets (buffer);
	if (*buffer)
		config.setFocusOffcenterDistance (buffer);

	printf ("Enter minFocusScore required after autofocus (0 if don't care) [%d]: ", config.minFocusScore());
	gets (buffer);
	if (*buffer)
		config.setMinFocusScore (buffer);

	config.saveToFile (configFilename);
	return 0;
}
