/* No representations are made concerning either the merchantability of
   this software or the suitability of this software for any particular
   purpose. It is provided "as is" without express or implied warranty
   of any kind.

   License to copy and use this software is granted provided that these
   notices are retained in any copies of any part of this documentation
   and/or software.
 */

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include "offset.h"

MicrolazeOffset::MicrolazeOffset ()
{
	_xOffset = 0.;
	_yOffset = 0.;
	_angleOffset = 0.;
}

// Return TRUE for success, FALSE if trouble opening file.
// Print warnings for errors in config file.
BOOL MicrolazeOffset::loadFromFile (char *filename)
{
	char buffer[80], *value;
	int line, i;
	
	FILE *infile = fopen (filename, "r");
	if (infile == NULL) {
		printf ("Cannot open offset file: %s\n", filename);
		return FALSE;
	}

	line = 0;
	while (1) {
		if (fgets (buffer, sizeof (buffer), infile) == NULL)
			break;
		++line;

		// Remove line terminators and spaces from end of string
		for (i = strlen (buffer) - 1; i >= 0; --i) {
			if (buffer[i] == '\012' || buffer[i] == '\015' || buffer[i] == ' ')
				buffer[i] = '\0';
			else
				break;
		}

		value = strchr (buffer, '=');
		if (value == NULL)
			// No '=' on line
			continue;

		// replace '=' with null termination so the parameter type is a normal string
		value[0] = '\0';
		// point value past the '='
		++value;

		if (stricmp (buffer, xOffsetName()) == 0)
			setXOffset (value);
		else if (stricmp (buffer, yOffsetName()) == 0)
			setYOffset (value);
		else if (stricmp (buffer, angleOffsetName()) == 0)
			setAngleOffset (value);
		else
			printf ("WARNING: offset line %d: unrecognized parameter %s\n", line, buffer);
	}

	fclose (infile);
	return TRUE;
}

// Return TRUE for success, FALSE if trouble opening file.
BOOL MicrolazeOffset::saveToFile (char *filename)
{
	FILE *outfile;

	outfile = fopen (filename, "w");
	if (outfile == NULL) {
		printf ("Cannot open offset file for write: %s\n", filename);
		return FALSE;
	}

	fprintf (outfile, "%s=%lf\n", xOffsetName(), xOffset());
	fprintf (outfile, "%s=%lf\n", yOffsetName(), yOffset());
	fprintf (outfile, "%s=%lf\n", angleOffsetName(), angleOffset());

	fclose (outfile);
	return TRUE;
}

// Reads a double from the value string.
// Prints warning if illegal value
void MicrolazeOffset::setXOffset (char *value)
{
	double doublevalue;

	if (sscanf (value, "%lf", &doublevalue) != 1)
		printf ("WARNING: offset: xoffset must be a number. Got: %s\n", value);
	else
		_xOffset = doublevalue;
}

// Reads a double from the value string.
// Prints warning if illegal value
void MicrolazeOffset::setYOffset (char *value)
{
	double doublevalue;

	if (sscanf (value, "%lf", &doublevalue) != 1)
		printf ("WARNING: offset: yoffset must be a number. Got: %s\n", value);
	else
		_yOffset = doublevalue;
}

// Reads a double from the value string.
// Prints warning if illegal value
void MicrolazeOffset::setAngleOffset (char *value)
{
	double doublevalue;

	if (sscanf (value, "%lf", &doublevalue) != 1)
		printf ("WARNING: offset: angleoffset must be a number. Got: %s\n", value);
	else
		_angleOffset = doublevalue;
}

