/* No representations are made concerning either the merchantability of
   this software or the suitability of this software for any particular
   purpose. It is provided "as is" without express or implied warranty
   of any kind.

   License to copy and use this software is granted provided that these
   notices are retained in any copies of any part of this documentation
   and/or software.
 */

static const double STEPS_PER_MM = 10000.0;

class Prior {
public:
	Prior ();
	Prior (BOOL doHome);
	~Prior ();

	void setQuiklaze (Quiklaze *quiklaze) {_quiklaze = quiklaze;}
	Quiklaze *quiklaze() {return _quiklaze;}
	void startFiring ();
	void stopFiring();

	double distanceToRefocus() {return _distanceToRefocus;}
	void setDistanceToRefocus (double distanceToRefocus);
	double focusOffcenterDistance() {return _focusOffcenterDistance;}
	void setFocusOffcenterDistance (double focusOffcenterDistance);
	double scoreStepSize() {return _scoreStepSize;}
	void setScoreStepSize (double scoreStepSize);
	double cutStepSize() {return _cutStepSize;}
	void setCutStepSize (double cutStepSize);
	double scoreHoldTime() {return _scoreHoldTime;}
	void setScoreHoldTime (double scoreHoldTime);
	double cutHoldTime() {return _cutHoldTime;}
	void setCutHoldTime (double cutHoldTime);
	int minFocusScore() {return _minFocusScore;}
	void setMinFocusScore (int minFocusScore);
	int cutFeedSpeed() {return _cutFeedSpeed;}
	void setCutFeedSpeed (int cutFeedSpeed);

	// x and y offset (mm) are the origin of the piece within the stage coordinates 
	double xOffset() {return _xOffset;}
	void setXOffset (double xOffset) {_xOffset = xOffset;}
	double yOffset() {return _yOffset;}
	void setYOffset (double yOffset) {_yOffset = yOffset;}
	// angle offset (radians, positive counterclockwise) is the rotation of the
	//   piece within the stage coordinates
	double angleOffset() {return _angleOffset;}
	void setAngleOffset (double angleOffset);

	void command (char *command);
	void command (char *command, char *buffer, DWORD bufferSize);
	BOOL reposition (double x, double y);
	BOOL cutTo (double x, double y);
    BOOL cutArcTo (double x, double y, double centerX, double centerY, BOOL clockwise);
	BOOL scoreTo (double x, double y);
    BOOL scoreArcTo (double x, double y, double centerX, double centerY, BOOL clockwise);
	void getXY (double *x, double *y);

	virtual BOOL checkInterrupt();
	virtual BOOL doInterrupt();

private:
	void initialize (BOOL doHome);
	BOOL checkFocus ();
	BOOL focus(double focusOffcenterDistance);
	void setSpeed (int speed);
	void moveTo (double x, double y);
	BOOL cutToHelper (double x, double y, double stepSize, double holdTime, int feedSpeed);
	BOOL cutArcToHelper 
     (double x, double y, double centerX, double centerY, BOOL clockwise, double stepSize,
      double holdTime, int feedSpeed);

	BOOL _doHome;
	Quiklaze *_quiklaze;
	HANDLE _hComm;
	int _speed;
	double _lastX, _lastY;
	double _distanceToRefocus;
	double _focusOffcenterDistance;
	double _focussedX, _focussedY;
	double _scoreStepSize, _cutStepSize;
	double _scoreHoldTime, _cutHoldTime;
	int _minFocusScore, _cutFeedSpeed;

	double _xOffset, _yOffset, _angleOffset, _cosAngle, _sinAngle;
};

