/* No representations are made concerning either the merchantability of
   this software or the suitability of this software for any particular
   purpose. It is provided "as is" without express or implied warranty
   of any kind.

   License to copy and use this software is granted provided that these
   notices are retained in any copies of any part of this documentation
   and/or software.
 */

#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <windowsx.h>
#include <time.h>
#include "serial.h"

/* Example, OpenLine (&hComm, "COM2", CBR_9600);
 */
int OpenLine(HANDLE *hComm, char *port, DWORD baud) 
{
	int result;
	DCB dcb;

#if 0
    // Overlapped version
	*hComm = CreateFile(port, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 
					 FILE_FLAG_OVERLAPPED, NULL);
#else
    // Non-overlapped version
	*hComm = CreateFile(port, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
#endif
	if (*hComm == INVALID_HANDLE_VALUE) return 0;
		result = GetCommState(*hComm, &dcb);				// retrieve com port info
	if (!result) return 0;
	
	dcb.BaudRate = baud;
	dcb.fParity = FALSE;
	dcb.Parity = NOPARITY;
	dcb.ByteSize = 8;
	dcb.StopBits = ONESTOPBIT;
	result = SetCommState(*hComm, &dcb);				// set baud rate
	if (!result) return 0;
		
	result = SetCommMask(*hComm, EV_RXCHAR);			// set to send an event if a char is received
	if (!result) return 0;

	return 1;
}

/* Read from COM port into lpBuf buffer which is of size lpBufSize.  Put the
      number of bytes read into bytes.
   Return TRUE for success, FALSE for error.
 */
BOOL ReadFromRS232(HANDLE hComm, unsigned char *lpBuf, DWORD lpBufSize, DWORD *bytes)
{
   // Non-overlapped version
   if (!ReadFile(hComm, lpBuf, lpBufSize, bytes, NULL))
	   return FALSE;

   return TRUE;
}

/* Write bytes of data from lpBuf to COM port
   Return TRUE for success, FALSE for error.
 */
BOOL WriteToRS232(HANDLE hComm, unsigned char *lpBuf, DWORD bytes)
{
#if 0
	// Overlapped version
	OVERLAPPED osWrite = {0};
	DWORD dwWritten;
	DWORD dwRes, dwToWrite;
	BOOL fRes;

	dwToWrite = bytes;
	// Create this write operation's OVERLAPPED structure's hEvent.
	osWrite.hEvent = CreateEvent(NULL, TRUE, FALSE,NULL);
	if (osWrite.hEvent == NULL)
		return FALSE;

	//Issue write
	if (!WriteFile(hComm, lpBuf, dwToWrite, &dwWritten, &osWrite)) {
		if (GetLastError() != ERROR_IO_PENDING) {
			//WriteFile failed, but isn't delayed. Report error and abort.
			fRes = FALSE;
		}
		else {
			//Write is pending.
			dwRes = WaitForSingleObject(osWrite.hEvent, INFINITE);
			switch(dwRes) {
				//OVERLAPPED structure's event has been signalled.
			case WAIT_OBJECT_0:
				if (!GetOverlappedResult(hComm, &osWrite, &dwWritten, FALSE))
					fRes = FALSE;
				else
					//Write operation completed successfully.
					fRes = TRUE;
				break;
			default:
				//An error has occurred in WaitForSingleObject.
				//This usually indicates a problem with the
				//OVERLAPPED structure's event handle.
				fRes = FALSE;
				break;
			}
		}
	}
	else
		//WriteFile comepleted immediately.
		fRes = TRUE;

	CloseHandle(osWrite.hEvent);
	return fRes;
#else
	// Non-overlapped version
	DWORD dwWritten;

	if (!WriteFile(hComm, lpBuf, bytes, &dwWritten, NULL))
			//WriteFile failed.
	  return (FALSE);

	return (TRUE);
#endif
}

/* Send buffer plus "\015" to the RS232.
   Return TRUE for success, FALSE for error.
 */
BOOL PutsRS232 (HANDLE hComm, char *buffer)
{
	if (WriteToRS232(hComm, (unsigned char *)buffer, strlen(buffer)) != TRUE)
		return FALSE;
	if (WriteToRS232(hComm, (unsigned char *)"\015", 1) != TRUE)
		return FALSE;

	return TRUE;
}

/* Read up the the first '\015' from the RS232, putting into buffer.
   Do not include the '\015' in the result.  Also, if the character preceding the CR
     is an LF, don't include it either.
   Add a terminating zero.
   buffer is of size bufferSize.
   Return TRUE for success, FALSE for error.
 */
BOOL GetsRS232 (HANDLE hComm, char *buffer, DWORD bufferSize)
{
	char c;
	DWORD bytes, dummy;

    bytes = 0;
	while (bytes < (bufferSize - 1)) {
	  if (ReadFromRS232(hComm, (unsigned char *)&c, 1, &dummy) != TRUE)
		return FALSE;

	  if (c == '\015')
		  break;

	  buffer[bytes++] = c;
	}

	if (bytes >= 1 && buffer[bytes - 1] == '\012')
	  --bytes;

	buffer[bytes] = 0;
	return TRUE;
}

// Pause the given number of seconds
void PauseSeconds (double seconds)
{
  time_t now, final;
  
  now = clock();
  final = (time_t)(now + seconds * CLOCKS_PER_SEC);

  while (clock() < final);
}

